from datetime import datetime
from elasticsearch_dsl import Document, Date, Nested, Boolean, InnerDoc, Completion, Keyword, Text, Integer, query
from elasticsearch_dsl import Search
from elasticsearch_dsl.connections import connections

index_name = "scrapy_douban_movie_comments"
client = connections.create_connection(hosts=["192.168.44.142"], http_auth=('elastic','elastic'))
# 创建document实例
douban_search = Search(using=client, index=index_name)


# 继承了es的Document
class DoubanCommentsType(Document):
    title = Text(analyzer="ik_max_word")
    url = Text()
    comments = Text(analyzer="ik_max_word")
    user = Keyword()
    vote = Integer()
    date = Date()
    createtime = Date()
    updatetime = Date()

    class Index:
        name = index_name
        settings = {
            "number_of_shards": 3,
            "number_of_replicas": 2
        }

    # 判断comments是否存在,返回True或False
    def exist_some_comments(self):
        print(f"exist_some_comments: {self.user}")
        s = douban_search.query("match", user=self.user)
        # 执行count查询,返回数字 https://elasticsearch-dsl.readthedocs.io/en/latest/api.html?highlight=count#elasticsearch_dsl.Search.count
        count = s.count()
        # 三元表达式，大于0返回True
        return True if count > 0 else False

    # 获取相同title的es源数据
    def get_some_comments(self):
        s = douban_search.query("match", user=self.user)
        # 执行搜索并返回Response包装所有数据的实例。
        res = s.execute()
        total = res.hits.total
        print('total hits', total.relation, total.value)
        # 这里的hits下的hits是es返回的josn格式。可以在kibana中执行scrapy_cnblogs/_search命令查看
        hits = res.hits.hits
        return hits;

    def mysave(self):
        if self.exist_some_comments() == True:
            print('更新comments,vote会有变化')
            hits = self.get_some_comments()
            self.meta.id = hits[0]["_id"]
            print(hits[0])
            self.createtime = hits[0]['_source']['createtime']
            self.updatetime = datetime.now()
            self.save()
        else:
            print('新增')
            # 如果文档不存在，则创建它，否则将覆盖它。
            self.createtime = datetime.now()
            self.save();


# init 创建索引并填充映射
if __name__ == '__main__':
    DoubanCommentsType.init()